/*
 * @ Author：lizhiyang1
 * @ Desc：通用文件上传。高版本浏览器用 formData ajax 上传；低版本用iframe form提交（暂不支持跨域）
 */

import $ from 'jquery';

const Upload = (($) => {

  const NAME = 'upload';
  const DATA_KEY = 'fuc.upload';
  const JQUERY_NO_CONFLICT = $.fn[NAME];

  class Upload {

    constructor(element, options) {
      this.defaults = {
        url: '',
        method: 'post',
        drag: false, //是否允许拖动上传，默认不允许
        thumbnail: false, //是否展示缩略图，默认false
        limitSize: 10240, //文件大小限制，默认10M
        limitNum: 1, //允许上传的文件数量，超出后不再显示增加按钮
        allowType: ['jpg', 'png'],
        onChange: null, //组件变化的回调，包括上传成功或删除上传的图片
        onStart: null, //开始上传的回调
        onEnd: null //上传结束的回调
      };
      this.options = $.extend({}, this.defaults, options);
      this.$element = $(element);
      this.$input = $(element).find('input[type="file"]');
      this.fileList = []; //上传成功的文件url列表
      this.attachEvent();
    }

    //绑定事件
    attachEvent() {
      const {drag, limitNum, onChange} = this.options;

      //透明的input标签
      this.$element.on('change', 'input[type="file"]', (e) => {
        const file = this.$input[0].files[0];
        this.init(file)
      });

      //拖拽事件
      drag && this.$element.on('drop', (e) => {
        e.preventDefault();
        const files = e.originalEvent.dataTransfer.files[0];
        this.init(files)
      });

      //删除按钮
      this.$element.on('click', '.delete', (e) => {
        const target = $(e.target);
        const item = target.parent('.item');
        const id = target.attr('data-id');
        this.fileList.splice(item.index(), 1);
        if (this.fileList.length < limitNum) {
          this.disabled(false);
        }
        target.parent('.item').remove();
        // onChange回调
        onChange && onChange(null, null, this.fileList)
      })
    }

    //文件校验
    init = (file) => {
      if (!file) return;
      const {size} = file;
      const {limitSize, thumbnail, allowType, onStart} = this.options;
      // 校验大小
      if (Math.ceil(size / 1024) > limitSize) {
        return this.handleUpload(`文件大小超限，请压缩至${limitSize / 1024}M以内`, null);
      }
      // 校验格式
      const type = file.type.split('/')[1];
      if (thumbnail && !/^(jpeg|jpg|png)$/i.test(type) || allowType.indexOf(type) === -1) {
        return this.handleUpload(`文件格式不正确，请上传${thumbnail ? '图片' : allowType.join(',')}格式的文件`, null);
      }
      onStart && onStart();
      // 判断是否支持FormData, 根据支持情况走不同方法
      window.FormData ? this.ajaxSubmit(file, this.handleUpload) : this.formSubmit(file, this.handleUpload);
    };

    //上传完成后的回调函数
    handleUpload = (err, res, file) => {
      const {thumbnail, limitNum, onChange, onEnd} = this.options;
      try {
        res = JSON.parse(res);
      } catch (e) {
        err = '接口返回内容解析错误';
        res = null;
      }
      //记录文件数量
      if (!err) {
        this.fileList.push(res.data);
        if (this.fileList.length >= limitNum) {
          this.disabled();
        }

        //插入dom
        const item = thumbnail ? `<div class="item"><img src="${res.data}"/><span class="delete" data-id="${res.data}"></span></div>` :
          `<div class="item">${file.name}<span class="delete" data-id="${res.data}">删除</span></div>`;
        this.$element.find('.file-list').append(item);
      }
      onEnd && onEnd();
      onChange && onChange(err, res, this.fileList);
    };

    //禁用
    disabled = (disabled = true) => {
      if (disabled) {
        this.$input.attr('disabled', 'disabled');
        this.$element.addClass('disabled');
      } else {
        this.$input.removeAttr('disabled');
        this.$element.removeClass('disabled');

      }
    };

    // 构建隐藏的iframe，通过form提交，适用于低版本浏览器，暂不支持跨域
    formSubmit = (file, cb) => {

      const {url, method} = this.options;
      let count = this.fileList.length;
      const iframeName = `upload_iframe_${count + 1}`;
      const iframe = $(`<iframe id="test" name="${iframeName}" style="display: none"></iframe>`);
      const formName = `form_${iframeName}`;
      const form = $(`<form name="${formName}" method="${method}" target="${iframeName}" action="${url}" style="display:none" enctype="multipart/form-data"/>`);
      form.append(this.$element.clone(true));

      $(document.body).append(iframe).append(form);
      //提交
      form.submit();
      iframe.on('load', function () {
        const contents = $(this).contents().get(0);
        const res = $(contents).find('body').text();
        cb(null, res, file);
        iframe.remove();
        form.remove();
      });
    };

    // HTML5 formData 上传
    ajaxSubmit = (file, cb) => {
      const {url, method} = this.options;
      const name = this.$input.attr('name');
      const data = new FormData();
      data.append(name, file);
      $.ajax({
        url,
        data,
        type: method,
        processData: false,
        mimeType: 'multipart/form-data',
        contentType: false, //multipart/form-data;
        success: (res) => {
          cb(null, res, file)
        },
        error: (err) => {
          cb(err, null, file)
        }
      })
    };

    // Static
    static _jQueryInterface(config) {
      return this.each(function () {
        let data = $(this).data(DATA_KEY);

        if (!data) {
          data = new Upload(this, config);
          $(this).data(DATA_KEY, data);
        }

        if (config === 'active') {
          data[config]();
        }
      });
    }
  }

  /**
   * ------------------------------------------------------------------------
   * jQuery
   * ------------------------------------------------------------------------
   */

  $.fn[NAME] = Upload._jQueryInterface;
  $.fn[NAME].Constructor = Upload;
  $.fn[NAME].noConflict = () => {
    $.fn[NAME] = JQUERY_NO_CONFLICT;
    return Upload._jQueryInterface;
  };

  return Upload;

})($);

export default Upload;
